#include "bpmplot.h"
#include <qutimearray3dplotplugin_i.h>
#include <QCheckBox>
#include <QPushButton>
#include <QGridLayout>
#include <QLabel>
#include <quwatcher.h>
#include <quplotcurve.h>
#include <cutango-world.h>

#include <QtDebug>

BpmPlot::BpmPlot(QWidget *parent, CumbiaPool *cp, const CuControlsFactoryPool &fp, QuTimeArray3DPlotPlugin_I *ta3dp)
    : QObject(parent), m_surface(nullptr), m_container(new QWidget(parent)), m_ta3dp(ta3dp), m_cu_p(cp), m_fpool(fp),
      m_history_siz(10) {
    QGridLayout *lo = new QGridLayout(m_container);
    QCheckBox *cb = new QCheckBox("Auto scale", m_container);
    connect(cb, SIGNAL(toggled(bool)), this, SLOT(setAutoscale(bool)));
    QLabel *lt = new QLabel("src", m_container);
    lt->setObjectName("ltitle");
    if(ta3dp)  {
        m_surface = ta3dp->create("QuTimeArray3DPlot", 0);
        m_surface->setFlags(m_surface->flags() ^ Qt::FramelessWindowHint);
        m_surface->setProperty("maxNumRows", m_history_siz);
        m_surface->resize(800, 800);
        m_plot = QWidget::createWindowContainer(m_surface, parent);
        QPushButton *pb3dconf = new QPushButton("Configure", m_container);
        connect(pb3dconf, SIGNAL(clicked()), this, SLOT(on3DConfClicked()));
        lo->addWidget(pb3dconf, 7, 0, 1, 1);
    }
    else {
        QuSpectrumPlot *p  = new QuSpectrumPlot(m_container, cp, fp);
        p->setYLowerBound(-8);
        p->setYUpperBound(8);
        m_plot = p;
    }
    lo->addWidget(lt, 0, 0, 1, 10);
    lo->addWidget(m_plot, 1, 0, 6, 10);
    lo->addWidget(cb, 7, 8, 1, 2);

    // Fill color map
    m_fill_colormap();
}

QWidget *BpmPlot::widget() {
    return m_container;
}

void BpmPlot::setSource(const QString &s) {
    printf("set source %s\n", qstoc(s));
    m_container->findChild<QLabel *>("ltitle")->setText(s.section('/', -1));
    if(m_ta3dp) {
        QuTimeArrayReader_I *reader = m_ta3dp->createReader(m_surface, m_cu_p, m_fpool);
        reader->setLink(s, m_surface);
    }
    else {
        QuWatcher *w = new QuWatcher(m_container, m_cu_p, m_fpool);
        connect(w, SIGNAL(newData(CuData)), this, SLOT(onUpdate(CuData)));
        w->setSource(s);
    }
}

void BpmPlot::setHistorySiz(int n) {
    m_history_siz = n;
    m_fill_colormap();
    if(m_surface)
        m_surface->setProperty("maxNumRows", m_history_siz);
}

void BpmPlot::onUpdate(const CuData &da) {
    bool e = da.B("err");
    CuVariant v = da["q"];
    //    printf("%s: q: %s msg '%s': %s\n", e ? "error" : "OK", da.s("qs").c_str(), da.s("msg").c_str(),  da.toString().c_str());

    m_container->setToolTip(da.s("msg").c_str());
    m_container->setEnabled(!e);
    if(m_ta3dp) {

    }
    else {
        QuSpectrumPlot *p = static_cast<QuSpectrumPlot *>(m_plot);
        QuPlotCurve *x = nullptr;
        int cc = p->curves().size();
        if(cc == m_history_siz) {
            cc--;
            // remove oldest curve
            x = m_crvmap[0];
            p->removeCurve(x->title().text()); // from plot
            m_crvmap.remove(0); // from map

            QList<int> keys = m_crvmap.keys();
            for(int i = 1; i < keys.size(); i++) {
                x = m_crvmap[i];
                if(i > 0) {
                    const int pi = i - 1;
                    // title: src + space + [idx]
                    const QString& src = x->title().text().section(' ', 0, 0);
                    x->setTitle(QString("%1 [%2]").arg(src).arg(pi)); // shift index in title
                    m_crvmap[pi] = x; // shift curve in map
                    x->setPen(m_colormap[pi]);
                    printf("crv %p %s picking color idx %d with green %d --> %d\n",
                           x, qstoc(x->title().text()), i - 1, m_colormap[pi].green(),
                           x->pen().color().green());
                }
            }

        }

        x = new QuPlotCurve(QString("%1 [%2]").arg(da.s("src").c_str()).arg(cc + 1));

        const std::vector<double>& y = da["value"].toDoubleVector();
        QVector<double> cx;
        for(size_t i = 0; i < y.size(); i++)
            cx << i;
        x->setData(cx, QVector<double>::fromStdVector(y));
        m_crvmap[cc] = x;
        p->addCurve(x->title().text(), x);
        x->setPen(QPen(m_colormap[cc]));
        p->refresh();
        printf("last curve %p %s \e[1;32mgreen %d\e[0m\n", x, qstoc(x->title().text()), x->pen().color().green());
    }
}

void BpmPlot::setAutoscale(bool a) {
    if(a) {
        if(m_ta3dp) {

        }
        else {
            QuSpectrumPlot *p = static_cast<QuSpectrumPlot *>(m_plot);
            p->setYAxisAutoscaleEnabled(true);
            p->refresh();
        }
    }
    else {
        if(m_ta3dp) {

        }
        else {
            QuSpectrumPlot *p = static_cast<QuSpectrumPlot *>(m_plot);
            p->setYLowerBound(-8);
            p->setYUpperBound(8);
            p->refresh();
        }
    }

}

void BpmPlot::on3DConfClicked() {
    if(m_ta3dp) {
        QWidget *w = m_ta3dp->plotConfigurationWidget(m_surface);
        w->setWindowFlag(Qt::Dialog);
        w->show();
    }
}

void BpmPlot::m_fill_colormap() {
    QColor c(66, 0, 236);
    // increase green up to 245
    // from dark blue to aqua
    int step = 245 / m_history_siz;
    for(int i = 0; i < m_history_siz; i++) {
        c.setGreen(c.green() + step);
        m_colormap[i] = c;
    }
}
